import sys
import json
import tempfile
from typing import Dict
from pathlib import Path

# Codigos de salida
SALIR_EXITO = 0
SALIR_ERROR = 1

# Carpetas globales
CARPETA_TEMPORAL = Path(tempfile.gettempdir())
CARPETA_ACTUAL = Path(__file__).parent.resolve()

# Archivo de ajustes
ARCHIVO_AJUSTES = CARPETA_ACTUAL / 'Ajustes.json'

# Se carga el archivo de ajustes en la variable AJUSTES
with open(ARCHIVO_AJUSTES, 'r') as archivo_json:
	AJUSTES: Dict[str, object] = json.load(archivo_json)

def error(mensaje: str, salir = True):
	''' Muestra un mensaje de error y opcionalmente termina el programa '''

	print(f'ERROR: {mensaje}')

	# Se termina el programa si se solicita
	if salir:
		sys.exit(SALIR_ERROR)

def ajuste(ruta_ajuste: str) -> object:
	''' Regresa el valor de un ajuste '''

	try:
		ajuste = AJUSTES
		ruta = ruta_ajuste.split('.')

		for campo in ruta:
			ajuste = ajuste[campo]

		return ajuste
	except:
		error(f'El ajuste \'{ruta_ajuste}\' no existe')

def ruta_kafka(servicio: str):
	''' Regresa el url del servicio de Kafka solicitado '''

	return ajuste(f'kafka.{servicio}')

# Carpetas generales
CARPETA_CSVDIFF     = CARPETA_ACTUAL / 'CSVDiff'
CARPETA_INSTALACION = Path(ajuste('instalacion')).expanduser()

# Archivos locales
ARCHIVO_SCRIPT     = CARPETA_ACTUAL / 'Script.py'
ARCHIVO_AJUSTES_PY = CARPETA_ACTUAL / 'Ajustes.py'
ARCHIVO_FILEPULSE  = CARPETA_ACTUAL / 'FilePulse.json'
ARCHIVO_COVID_BASE = CARPETA_ACTUAL / 'datos_abiertos_20000101.zip'

# Archivos instalación
INSTALACION_BUILD       = CARPETA_INSTALACION / 'build'
INSTALACION_CSVDIFF     = INSTALACION_BUILD / 'CSVDiff'
INSTALACION_SCRIPT      = CARPETA_INSTALACION / 'Script.py'
INSTALACION_DATOS_COVID = CARPETA_INSTALACION / 'datos_covid'
INSTALACION_DIFERENCIAS = CARPETA_INSTALACION / 'diferencias'