# Desactiva el bytecode de Python
import sys
sys.dont_write_bytecode = True

# Librerías
import shutil
import getpass
import subprocess
import importlib.util

# Ajustes
from Ajustes import *

def comprobar_comando(comando: str, sugerencia: str = None):
	''' Comprueba que un comando esté instalado '''

	if shutil.which(comando) is None:
		if sugerencia is not None:
			print(f'Se recomienda el comando \'{sugerencia}\'')
		error(f'\'{comando}\' no está instalado')

def instalar_paquete(paquete: str, nombre_import: str = None):
	''' Instala un paquete de Python si no está instalado '''

	# El nombre del paquete de importación suele ser el mismo que el nombre del paquete
	if nombre_import is None:
		nombre_import = paquete

	# Se comprueba que el paquete esté instalado
	if importlib.util.find_spec(nombre_import) is None:
		print(f'El paquete \'{paquete}\' no está instalado. Instalando...')
		sub = subprocess.run(['pip3', 'install', paquete],
			stdout = subprocess.DEVNULL, stderr = subprocess.STDOUT)

		if sub.returncode != 0:
			error(f'No se pudo instalar el paquete \'{paquete}\'')

def crear_carpeta(carpeta: Path):
	''' Crea una carpeta si no existe '''

	if not carpeta.exists():
		print(f'Creando carpeta \'{carpeta}\'...')
		carpeta.mkdir(parents = True)

def copiar_archivo(archivo: Path, destino: Path):
	''' Copia un archivo a una carpeta destino '''

	print(f'Copiando \'{archivo}\' a \'{destino}\'...')
	shutil.copy(str(archivo), str(destino))

def compilar_csvdiff():
	''' Compila el programa CSVDiff '''

	print('Compilando CSVDiff...')
	sub_cmake = subprocess.run(['cmake', '-S', str(CARPETA_CSVDIFF), '-B', str(INSTALACION_BUILD)],
		stdout = subprocess.DEVNULL, stderr = subprocess.STDOUT)
	sub_make = subprocess.run(['make', '-C', str(INSTALACION_BUILD)],
		stdout = subprocess.DEVNULL, stderr = subprocess.STDOUT)

	# Se comprueba que la compilación haya sido exitosa
	if sub_cmake.returncode != SALIR_EXITO or sub_make.returncode != SALIR_EXITO:
		error('No se pudo compilar CSVDiff')

def crear_topico():
	''' Crea el tópico de Kafka '''

	import requests
	print('Creando tópico de Kafka...')

	# Se manda una solicitud para consultar los tópicos de Kafka
	respuesta = requests.get(ruta_kafka('rest_proxy') + '/topics')

	# Se comprueba la respuesta de la solicitud
	if not respuesta.ok:
		error('No se pudiieron consultar los tópicos de Kafka')

	# Solo se crea el tópico si no existe
	if ajuste('kafka.topic') not in respuesta.json():

		# Se manda una solicitud para obtener el cluster de Kafka
		respuesta = requests.get(ruta_kafka('rest_proxy') + '/v3/clusters')

		# Se comprueba la respuesta de la solicitud
		if not respuesta.ok:
			error('No se pudo obtener el cluster de Kafka')

		# Se obtiene el id del cluster
		cluster_id = respuesta.json()['data'][0]['cluster_id']

		# Se manda una solicitud para crear el tópico
		respuesta = requests.post(
			ruta_kafka('rest_proxy') + '/v3/clusters/' + cluster_id + '/topics',
			json = {'topic_name': ajuste('kafka.topic')})

		# Se comprueba la respuesta de la solicitud
		if not respuesta.ok:
			error('No se pudo crear el tópico de Kafka')

def cambiar_retencion():
	''' Modifica el tiempo de retención del tópico de Kafka '''

	import requests
	print('Modificando retención del tópico de Kafka...')

	# Se manda una solicitud para obtener el cluster de Kafka
	respuesta = requests.get(ruta_kafka('rest_proxy') + '/v3/clusters')

	# Se comprueba la respuesta de la solicitud
	if not respuesta.ok:
		error('No se pudo obtener el cluster de Kafka')

	# Se obtiene el id del cluster
	cluster_id = respuesta.json()['data'][0]['cluster_id']

	# Se manda una solicitud para modificar el tiempo de retención del tópico
	respuesta = requests.put(ruta_kafka('rest_proxy') + '/v3/clusters/' +
		cluster_id + '/topics/' + ajuste('kafka.topic') + '/configs/retention.ms',
		json = {'value': ajuste('kafka.retencion')})

	# Se comprueba la respuesta de la solicitud
	if not respuesta.ok:
		error('No se pudo modificar el tiempo de retención del tópico de Kafka')

def crear_conector_filepulse():
	''' Crea el conector de FilePulse '''

	import requests
	print('Creando conector de FilePulse...')

	# Se modifica el archivo de configuración de FilePulse
	with open(ARCHIVO_FILEPULSE, 'r') as archivo_filepulse:
		ajustes = archivo_filepulse.read()
		ajustes = ajustes.replace('{NOMBRE_TOPICO}', ajuste('kafka.topic'))
		ajustes = ajustes.replace('{BOOTSTRAP_SERVER}', ruta_kafka('bootstrap_server'))
		ajustes = ajustes.replace('{CARPETA_DIFERENCIAS}', str(INSTALACION_DIFERENCIAS))

		# Se manda una solicitud POST a Kafka Connect para crear el conector
		respuesta = requests.post(
			ruta_kafka('kafka_connect') + '/connectors',
			json = {'name': ajuste('kafka.conector'), 'config': json.loads(ajustes)})

		# Se comprueba la respuesta de la solicitud
		if not respuesta.ok:
			error('No se pudo crear el conector de FilePulse')

def crear_cronjob():
	''' Crea el cronjob para ejecutar el script diariamente '''

	import crontab
	print('Creando cronjob...')

	# Ajustes
	NOMBRE = ajuste('cronjob.nombre')
	HORA = int(ajuste('cronjob.hora').split(':')[0])
	MINUTO = int(ajuste('cronjob.hora').split(':')[1])

	# Se obtiene la ruta del ejecutable de Python3
	sub = subprocess.run(['whereis', 'python3'], capture_output = True)

	# Se crea el cronjob si se pudo obtener la ruta de Python3
	if sub.returncode == SALIR_EXITO:
		cron = crontab.CronTab(user = getpass.getuser())
		ruta_python3 = sub.stdout.decode('utf-8').split(' ')[1]
		tarea = cron.new(f'{ruta_python3} {INSTALACION_SCRIPT} 2>&1 | logger -t {NOMBRE}')
		tarea.hour.on(HORA); tarea.minute.on(MINUTO)
		cron.write()
	else:
		error('No se pudo crear el cronjob')

# Se comprueban los comandos de sistema
comprobar_comando('pip3', 'sudo apt install python3-pip')
comprobar_comando('gcc', 'sudo apt install build-essential')
comprobar_comando('make', 'sudo apt install build-essential')
comprobar_comando('cmake', 'sudo apt install cmake')

# Se instalan los paquetes de Python
instalar_paquete('requests')
instalar_paquete('zipfile_deflate64')
instalar_paquete('python-crontab', 'crontab')

# Se crean las carpetas de instalación
crear_carpeta(INSTALACION_BUILD)
crear_carpeta(INSTALACION_DATOS_COVID)
crear_carpeta(INSTALACION_DIFERENCIAS)

# Se copian los archivos de instalación
copiar_archivo(ARCHIVO_SCRIPT, CARPETA_INSTALACION)
copiar_archivo(ARCHIVO_AJUSTES, CARPETA_INSTALACION)
copiar_archivo(ARCHIVO_AJUSTES_PY, CARPETA_INSTALACION)
copiar_archivo(ARCHIVO_COVID_BASE, INSTALACION_DATOS_COVID)

# Se compila CSVDiff
compilar_csvdiff()

# Se crea el conector de FilePulse
crear_conector_filepulse()

# Se crea el cronjob del script
crear_cronjob()