# Tópico de Kafka para Datos de COVID-19

## Descripción

Esta carpeta contiene todo lo necesario para:
  - Bajar diariamente el archivo actualizado con los casos de COVID-19 desde el sitio de la Dirección General de Epidemiología.
  - Obtener las diferencias entre registros entre el archivo del día anterior y el del día actual mediante el programa **CSVDiff**.
  - Cargar los datos de las diferencias en un tópico de Kafka mediante un conector de **FilePulse**.

En resumen, se genera un tópico de Apache Kafka que contiene los datos de los casos de COVID-19 en México. Los datos se actualizan diariamente de manera automática y pueden ser usados por cualquier aplicación que consuma datos de Kafka para tener siempre los datos actualizados evitando así hacer un proceso de ETL.

## Requisitos

Para poder ejecutar el proceso de carga de datos en Kafka es necesario tener instalados los siguientes paquetes en el sistema donde se ejecutará el proceso:
  - GCC y Make
  - CMake
  - Python 3 y Pip

En el mismo sistema donde se ejecuta el proceso se debe tener instalado y configurado un servicio de **Confluent Kafka** con el siguiente conector:
  - FilePulse 2.7.0

## Instalación

Para facilitar la implementación del proceso de carga de datos en Kafka, se incluye un script de instalación que se encarga de configurar todo el proceso de manera automática. El script realiza las siguientes tareas:

- Comprobar que se tenga instalado:
  - pip3
  - gcc
  - make
  - cmake
- Instalar los paquetes de Python 3:
  - requests
  - zipfile_deflate64
  - python-crontab
- Crear la carpeta de instalación
- Copiar los archivos necesarios
- Compilar el programa **CSVDiff**
- Crear el conector de **FilePulse**
- Crear la tarea periódica de carga de datos

Para ejecutar el script de instalación se debe ejecutar el siguiente comando:

```sh
python3 Instalar.py
```

# Desinstalación

Para desinstalar el proceso de carga de datos en Kafka, se incluye un script de desinstalación que se encarga de eliminar todo lo que se creó durante la instalación. El script realiza las siguientes tareas:

- Se elimina la carpeta de instalación y su contenido
- Se elimina la tarea periódica de carga de datos
- Se elimina el conector de **FilePulse**
- Se eliminan los paquetes de Python 3

El script de desinstalación se ejecuta con el siguiente comando:

```sh
python3 Desinstalar.py
```

**Nota:** El script de desinstalación no elimina el tópico de Kafka donde se cargan los datos.

## Ajustes

Todos los ajustes del proceso de carga de datos se encuentran en el archivo **Ajustes.json**. Este archivo contiene los siguientes campos:

- **url_datos_dge**: URL del archivo de datos de COVID-19 de la Dirección General de Epidemiología.
- **instalacion**: Carpeta de instalación del proceso de carga de datos.
- **cronjob**:
  - **nombre**: Nombre con el que se identificará la tarea periódica en **var/log/syslog**.
  - **hora**: Hora en la que se ejecutará la tarea periódica.
- **dia_especial**:
  - **autodetectar**: Indica si se debe autodetectar el día especial. Los días especiales son aquellos donde se omite una gran cantidad de registros que estaban presentes en archivos anteriores.
  - **eliminados**: Cantidad de registros eliminados entre el archivo del día anterior y el del día actual para considerarlo como un día especial.
- **kafka**:
  - **bootstrap_server**: Servidor donde se encuentra el servicio de **Confluent Kafka**.
  - **kafka_connect**: Servidor donde se encuentra el servicio de **Kafka Connect**.
  - **rest_proxy**: Servidor donde se encuentra el servicio de **REST Proxy**.
  - **conector**: Nombre del conector de **FilePulse**.
  - **topic**: Nombre del tópico de Kafka donde se cargarán los datos.
  - **retencion**: Tiempo en milisegundos de retención de los datos en el tópico de Kafka. (**-1** para nunca eliminar los datos)

# El script de diferencias

Una vez que se ha instalado el proceso de carga de datos en Kafka, es el archivo **Script.py** el cual se ejecuta diariamente para obtener las diferencias entre el archivo del día anterior y el del día actual. Esas diferencias son automáticamente consumidas por el conector de **FilePulse** y cargadas en el tópico de Kafka.

Este archivo tiene una interfaz de línea de comandos que permite ejecutarlo de manera manual para obtener las diferencias entre dos archivos. La sintaxis es la siguiente:

```sh
python3 Script.py [-h] [-b BASE] [-d DIFF] [-s] [-n]

options:
  -h, --help            show this help message and exit
  -b BASE, --base BASE  Archivo base
  -d DIFF, --diff DIFF  Archivo diff
  -e, --especial        Dia especial
  -n, --no-descargar    No descargar archivo de datos del día actual
```

**Nota:** Es posible que el script de diferencias falle debido a que el conector de **FilePulse** no haya terminado de cargar los registros del día anterior. En ese caso, el script reiniciará el conector de **FilePulse** y volverá a intentar la carga de datos el día siguiente. Esto provocará dos cosas:
- El tópico de Kafka se actualizará con un día de retraso.
- Faltará un día de datos en el tópico de Kafka.

# Carga de datos (sin inicialización)

Si no se desea cargar el conjunto de datos completo de la DGE, se puede simplemente dejar que se ejecute el script de diferencias de manera automática sin hacer ningún cambio, de tal manera que el tópico de Kafka se llenará únicamente con los registros más nuevos que haya en el archivo de datos de la DGE.

# Carga inicial de datos (Tópico vacío)

Esta sección aplica cuando se tiene un tópico de Kafka vacío y se desea llenar con el conjunto completo de datos de COVID-19 de la DGE.

**Nota:** Este proceso se debe realizar antes de que se ejecute el script de diferencias por primera vez, lo cual debe ser antes del tiempo especificado en **Ajustes.json** en el campo **cronjob.hora**.

## Primer día especial (18/08/2022)

Se debe llenar el tópico de kafka con un archivo que contenga todos los registros previos al día especial conocido del **18/08/2022**, para ello es necesario conseguir el archivo **datos_abiertos_20220818.zip** y colocarlo en la carpeta **instalacion/datos_covid**. Una vez que se tenga el archivo, se puede correr el script de diferencias con la siguiente sintaxis:

```sh
python3 Script.py -n
```

**Importante:** Después de ejecutar este comando, es necesario esperar que termine la carga de datos en el tópico de Kafka. Esto se puede verificar comprobando que la carpeta **instalacion/diferencias** se encuentre vacía.

## Segundo día especial (31/05/2023)

Después de cargar los datos anteriores, existe un segundo día especial conocido del **31/05/2023**. Para cargar los datos de este día, se debe conseguir el archivo **datos_abiertos_20230531.zip** y colocarlo en la carpeta **instalacion/datos_covid**. Una vez que se tenga el archivo, se puede correr el script de diferencias con la siguiente sintaxis:

```sh
python3 Script.py -n -e
```

**Importante:** Después de ejecutar este comando, es necesario esperar que termine la carga de datos en el tópico de Kafka. Esto se puede verificar comprobando que la carpeta **instalacion/diferencias** se encuentre vacía.

## Actualización diaria

Finalmente, se debe correr el script de diferencias de manera normal asegurándose de marcar la opción **-e** para que se detecte el día especial cargado en el paso anterior y se omitan los registros eliminados:

```sh
python3 Script.py -e
```

Una vez que finalice la ejecución del script, se habrá cargado el tópico de Kafka con el conjunto de datos más reciente de todos los registros de COVID-19 y se actualizará diariamente de manera automática.

# Carga inicial de datos (Tópico con datos)

Esta sección aplica cuando se tiene un tópico de Kafka con datos ya cargados de COVID-19 y se desea que se actualice con datos nuevos de manera automática.

**Nota:** Este proceso se debe realizar antes de que se ejecute el script de diferencias por primera vez, lo cual debe ser antes del tiempo especificado en **Ajustes.json** en el campo **cronjob.hora**.

**Importante:** Los registros del tópico de Kafka deben contener todos los campos originales del archivo .csv descargado de la DGE mas un campo **TYPE** que indique si el registro es una inserción, actualización o eliminación (**INSERT**, **UPDATE** o **DELETE**). Si los registros no contienen este campo, cualquier conector que consuma datos asumiendo este formato fallará.

## Instrucciones

Todo lo que se tiene que hacer es conseguir el último archivo de datos con formato **datos_abiertos_AAAAMMDD.zip** que se haya usado para llenar el tópico de Kafka y colocarlo en la carpeta **instalacion/datos_covid**, después de eso, el script de diferencias se encargará de hacer el resto de manera automática.