# Tópico de Kafka para Datos de COVID-19

## Descripción

Esta carpeta contiene todo lo necesario para:
  - Bajar diariamente el archivo actualizado con los casos de COVID-19 desde el sitio de la Dirección General de Epidemiología.
  - Obtener las diferencias entre registros entre el archivo del día anterior y el del día actual mediante el programa **CSVDiff**.
  - Cargar los datos de las diferencias en un tópico de Kafka mediante un conector de **FilePulse**.

En resumen, se genera un tópico de Apache Kafka que contiene los datos de los casos de COVID-19 en México. Los datos se actualizan diariamente de manera automática y pueden ser usados por cualquier aplicación que consuma datos de Kafka para tener siempre los datos actualizados evitando así hacer un proceso de ETL.

## Requisitos

Para poder ejecutar el proceso de carga de datos en Kafka es necesario tener instalados los siguientes paquetes en el sistema donde se ejecutará el proceso:
  - GCC y Make
  - CMake
  - Python 3 y Pip

En el mismo sistema donde se ejecuta el proceso se debe tener instalado y configurado un servicio de **Confluent Kafka** con el siguiente conector:
  - FilePulse 2.7.0

## Instalación

Para facilitar la implementación del proceso de carga de datos en Kafka, se incluye un script de instalación que se encarga de configurar todo el proceso de manera automática. El script realiza las siguientes tareas:

- Comprobar que se tenga instalado:
  - pip3
  - gcc
  - make
  - cmake
- Instalar los paquetes de Python 3:
  - requests
  - zipfile_deflate64
  - python-crontab
- Crear la carpeta de instalación
- Copiar los archivos necesarios
- Compilar el programa **CSVDiff**
- Crear el conector de **FilePulse**
- Crear la tarea periódica de carga de datos

Para ejecutar el script de instalación se debe ejecutar el siguiente comando:

```sh
python3 Instalar.py
```

# Desinstalación

Para desinstalar el proceso de carga de datos en Kafka, se incluye un script de desinstalación que se encarga de eliminar todo lo que se creó durante la instalación. El script realiza las siguientes tareas:

- Se elimina la carpeta de instalación y su contenido
- Se elimina la tarea periódica de carga de datos
- Se elimina el conector de **FilePulse**
- Se eliminan los paquetes de Python 3

El script de desinstalación se ejecuta con el siguiente comando:

```sh
python3 Desinstalar.py
```

**Nota:** El script de desinstalación no elimina el tópico de Kafka donde se cargan los datos.

## Ajustes

Todos los ajustes del proceso de carga de datos se encuentran en el archivo **Ajustes.json**. Este archivo contiene los siguientes campos:

- **url_datos_dge**: URL del archivo de datos de COVID-19 de la Dirección General de Epidemiología.
- **instalacion**: Carpeta de instalación del proceso de carga de datos.
- **cronjob**:
  - **nombre**: Nombre con el que se identificará la tarea periódica en **var/log/syslog**.
  - **hora**: Hora en la que se ejecutará la tarea periódica.
- **dia_especial**:
  - **autodetectar**: Indica si se debe autodetectar el día especial. Los días especiales son aquellos donde se omite una gran cantidad de registros que estaban presentes en archivos anteriores.
  - **eliminados**: Cantidad de registros eliminados entre el archivo del día anterior y el del día actual para considerarlo como un día especial.
- **kafka**:
  - **servidor**: Servidor donde se encuentra el servicio de **Confluent Kafka**.
  - **conector**: Nombre del conector de **FilePulse**.
  - **topic**: Nombre del tópico de Kafka donde se cargarán los datos.
  - **puertos**:
    - **bootstrap_server**: Puerto del servicio de **Confluent Kafka**.
    - **kafka_connect**: Puerto del servicio de **Kafka Connect**.
    - **rest_proxy**: Puerto del servicio de **REST Proxy**.

# El script de diferencias

Una vez que se ha instalado el proceso de carga de datos en Kafka, es el archivo **Script.py** el cual se ejecuta diariamente para obtener las diferencias entre el archivo del día anterior y el del día actual. Esas diferencias son automáticamente consumidas por el conector de **FilePulse** y cargadas en el tópico de Kafka.

Este archivo tiene una interfaz de línea de comandos que permite ejecutarlo de manera manual para obtener las diferencias entre dos archivos. La sintaxis es la siguiente:

```sh
python3 Script.py [-h] [-b BASE] [-d DIFF] [-s] [-n]

options:
  -h, --help            show this help message and exit
  -b BASE, --base BASE  Archivo base
  -d DIFF, --diff DIFF  Archivo diff
  -e, --especial        Dia especial
  -n, --no-descargar    No descargar archivo de datos del día actual
```

**Nota:** Es posible que el script de diferencias falle debido a que el conector de **FilePulse** no haya terminado de cargar los registros del día anterior. En ese caso, el script reiniciará el conector de **FilePulse** y volverá a intentar la carga de datos el día siguiente. Esto provocará dos cosas:
- El tópico de Kafka se actualizará con un día de retraso.
- Faltará un día de datos en el tópico de Kafka.

## Carga inicial de datos

Para poder comenzar con la carga normal de los datos, es necesario inicializar el tópico de Kafka con un archivo que contenga todos los registros previos al día especial conocido del **18/08/2022**, para ello es necesario conseguir el archivo **datos_abiertos_20220818.zip** y colocarlo en la carpeta **instalacion/datos_covid**. Una vez que se tenga el archivo, se puede correr el script de diferencias con la siguiente sintaxis:

```sh
python3 Script.py -n
```

Después de ejecutar este comando, es necesario esperar que termine la carga de datos en el tópico de Kafka. Esto se puede verificar comprobando que la carpeta **instalacion/diferencias** se encuentre vacía.

Posteriormente, se debe correr el script de diferencias de manera normal asegurándose de marcar la opción **-s** para que se detecte el día especial y se omitan los registros eliminados:

```sh
python3 Script.py -e
```

Con esto, se habrá cargado el tópico de Kafka con la lista más reciente de todos los registros de COVID-19 y se actualizará diariamente de manera automática.