# Desactiva el bytecode de Python
import sys
sys.dont_write_bytecode = True

# Librerías
import glob
import argparse
import requests
import subprocess
from os import cpu_count
from datetime import date
import zipfile_deflate64 as zipfile

# Ajustes
from Ajustes import *

# Interfaz de línea de comandos
parser = argparse.ArgumentParser()
parser.add_argument('-b', '--base', help='Archivo base', type=str)
parser.add_argument('-d', '--diff', help='Archivo diff', type=str)
parser.add_argument('-e', '--especial', help='Dia especial', action='store_true')
parser.add_argument('-n', '--no-descargar', help='No descargar archivo de datos del día actual', action='store_true')
parser.add_argument('-f', '--forzar_actual', help='Inicializa el tópico de Kafka con el archivo mas reciente', action='store_true')
argumentos = parser.parse_args()

# Variables de la interfaz de línea de comandos
DIA_ESPECIAL: bool = argumentos.especial
DESCARGAR_ACTUAL: bool = not argumentos.no_descargar
FORZAR_ACTUAL: bool = argumentos.forzar_actual

# Se comprueba si existe el tópico de Kafka
respuesta = requests.get(ruta_kafka('rest_proxy') + '/topics/' + ajuste('kafka.topic'))

# Si no existe el tópico y no se especificó un archivo base o diff, se termina el script
if not respuesta.ok and argumentos.base is None and argumentos.diff is None:

	# Se omite el error si no se descarga el archivo de datos del día actual
	if DESCARGAR_ACTUAL:
		error('No se ha inicializado el tópico de Kafka')

# Se genera el nombre del archivo de datos del día actual
ARCHIVO_DIA_ACTUAL = 'datos_abiertos_' + date.today().strftime('%Y%m%d') + '.zip'

# Solo se descarga el archivo de datos del día actual si no existe
if DESCARGAR_ACTUAL and not (INSTALACION_DATOS_COVID / ARCHIVO_DIA_ACTUAL).exists():

	# Se descarga el archivo de datos del día actual
	sub = subprocess.run(['wget', '-O',
		str(INSTALACION_DATOS_COVID / ARCHIVO_DIA_ACTUAL),
		ajuste('url_datos_dge')])

	# Se comprueba que la descarga haya sido exitosa
	if sub.returncode != SALIR_EXITO:
		error(f'No se pudo descargar el archivo de datos del día actual')

# Se obtiene una lista de todos los archivos de datos
ARCHIVOS_COVID = sorted(glob.glob(f'{INSTALACION_DATOS_COVID}/datos_abiertos_*.zip'))

# Se obtiene el archivo base y el archivo de diferencias
ARCHIVO_BASE = ARCHIVOS_COVID[-2] if argumentos.base is None else argumentos.base
ARCHIVO_DIFF = ARCHIVOS_COVID[-1] if argumentos.diff is None else argumentos.diff

# Archivos de diferencias
ARCHIVO_INSERT = CARPETA_TEMPORAL / 'insert.csv'
ARCHIVO_DELETE = CARPETA_TEMPORAL / 'delete.csv'
ARCHIVO_UPDATE = CARPETA_TEMPORAL / 'update.csv'

# Se descomprime el archivo base
with zipfile.ZipFile(ARCHIVO_BASE, 'r') as archivo_zip:
	ARCHIVO_BASE_TEMPORAL = CARPETA_TEMPORAL / archivo_zip.filelist[0].filename
	archivo_zip.extractall(CARPETA_TEMPORAL)

# Se descomprime el archivo diff
with zipfile.ZipFile(ARCHIVO_DIFF, 'r') as archivo_zip:
	ARCHIVO_DIFF_TEMPORAL = CARPETA_TEMPORAL / archivo_zip.filelist[0].filename
	archivo_zip.extractall(CARPETA_TEMPORAL)

# Se hace la diferencia de ambos archivos
sub = subprocess.run(
[
	INSTALACION_CSVDIFF, '-cfu',

	# Se usan todos los núcleos disponibles
	'-t', str(cpu_count()),

	# La columna con índice 1 (ID_REGISTRO) es la llave primaria
	'-p', '1',

	# La columna con índice 0 (FECHA_ACTUALIZACION) se ignora
	'-i', '0',

	# Ubicación de los archivos de diferencias
	'-I', str(ARCHIVO_INSERT),
	'-D', str(ARCHIVO_DELETE),
	'-U', str(ARCHIVO_UPDATE),

	# Archivo base y de diferencias
	str(ARCHIVO_BASE_TEMPORAL),
	str(ARCHIVO_DIFF_TEMPORAL)
])

# Se borran los archivos temporales
ARCHIVO_BASE_TEMPORAL.unlink()
ARCHIVO_DIFF_TEMPORAL.unlink()

# Se comprueba que la diferencia haya sido exitosa
if sub.returncode != SALIR_EXITO:
	error('No se pudo hacer la diferencia de los archivos')

# Se autodetecta el día especial
if ajuste('dia_especial.autodetectar') and not DIA_ESPECIAL:
	with open(ARCHIVO_DELETE) as archivo:
		cantidad_eliminados = sum(1 for _ in archivo)

	# Si se sobrepasa el límite de eliminados, se marca como día especial
	if cantidad_eliminados > ajuste('dia_especial.eliminados'):
		print('Se detecto un día especial')
		DIA_ESPECIAL = True

# Se asegura que la carpeta de diferencias este vacía
if any(INSTALACION_DIFERENCIAS.iterdir()):
	print('Se detectaron archivos de diferencias previos')

	# Se manda una solicitud POST a Kafka Connect para reiniciar el conector
	respuesta = requests.post(ruta_kafka('kafka_connect') +
		'/connectors/' + ajuste('kafka.conector') + '/restart')

	# Se comprueba la respuesta de la solicitud
	if not respuesta.ok:
		error('No se pudo reiniciar el conector de FilePulse')

	# Se termina el script con un error
	error('La carpeta de diferencias no esta vacia')
else:

	# Se mueven los archivos de diferencias
	ARCHIVO_INSERT.rename(INSTALACION_DIFERENCIAS / ARCHIVO_INSERT.name)
	ARCHIVO_UPDATE.rename(INSTALACION_DIFERENCIAS / ARCHIVO_UPDATE.name)

	# El archivo de registros eliminados solo se mueve si no es día especial
	if not DIA_ESPECIAL:
		ARCHIVO_DELETE.rename(INSTALACION_DIFERENCIAS / ARCHIVO_DELETE.name)
	else:
		ARCHIVO_DELETE.unlink()